#|

This file contains a set of functions that are called from SD Dialogs
to build parts like bushings, dowels, etc.
Each function should be documented with it's purpose as well as
all of the Dialogs that use it.  These are functions for creating parts.

To make use of these functions, the calling Dialogs must use the 
in-package LHC.

11-Nov-99  Added BOM attributes 

12-Nov-99  Found that these operations only work with the active part.  The dialog allows that part to be changed, but it has no effect.  The
fix is to add an "after-input" statement to the dialog macro.

Future enhancements     : 

|#

;; Last modified    : 11-Nov-1999
;; Category         : Parts
;; Author           : Bruce Fisher (bfisher@lhcarbide.com)
;; Company          : L.H. Carbide Corporation
;; Filename         : functions_2.lsp
;; Calling Dialogs  : 
;; SD Revision      : 7.11,7.51
;; Platform         : Windows NT 4.0

(in-package :LHC)
(use-package :OLI)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Last modified    : 13-Nov-1999
;; Category         : Parts
;; Company          : L.H. Carbide Corporation
;; Author           : Bruce Fisher
;; Function Name    : make_bshg
;; Parameters       : Location point, Length of Cylinder, O.D., I.D.
;; Purpose          : create a bushing at selected location
;; SD Revision      : 7.51
;; Funct. Revision  : 1.30
;; Calling Dialogs  : lhc_mk_bshg
;; Comments         : Creates back taper at .0015 P.I.P.S. (Could be a parameter)

 (defun make_bshg (A_PRT P1 DP1 DIA_1 DIA_2 CUR_ASM B_NME)
   (setq curr_wp (sd-inq-curr-wp))
   (setq curr_prt a_prt)
   (setq pr1 (/ DIA_2 2)) ;calc radius for I.D. at top
   (setq pr2 (/ DIA_1 2)) ;calc radius for O.D.
   (setq pr3 (- pr2 .001)) ;calc start diameter
   (setq pr4 (+ (* .0015 DP1)(/ DIA_2 2))) ;calculate radius for I.D. at bottom
   (setq pr6 (* -1 dp1)) ;calc length of bshg
   (setq pr5 (+ pr6 .12)) ;calc length of bshg - start length of start dia.

     (sd-call-cmds 
       (CREATE_WORKPLANE 
                         :copy
                         :name "mk_bshg.wp"
                         :source curr_wp)) 

;; The newly created workplane is created under the root by default so
;; the root "/" has to be part of the workplane name.  It is desirable
;; to slide the origin of the workplane before rotating it so that the
;; machining operation is easy to calculate.

    (slide_wp_origin (sd-inq-curr-wp) p1) 
    (position_wp (sd-inq-curr-wp)
           :rotate
           :axis
           :u
           :rotation_angle 90.0
           :done 
      )

;; Now build the bushing profile from user inputs

    (del_2d_geo (sd-inq-curr-wp))  ;get rid of baggage profiles first 

   (polygon
    (gpnt2d pr1 0)
    (gpnt2d pr2 0)
    (gpnt2d pr2 pr5)
    (gpnt2d pr3 pr5) 
    (gpnt2d pr3 pr6)
    (gpnt2d pr4 pr6)
    :close)

;; Create a new part before machining action.  
;; Need to create a unique name here.  Possibly with another function

     (sd-call-cmds 
       (CREATE_PART 
                         :name b_nme
                         :owner cur_asm   )) 

;; Run the turn tool.  

    (sd-call-cmds
    (turn  :parts b_nme
           :axis 
           :v
           :rotation_angle (* pi 2)
           :keep_wp :no
           :keep_profile :no)
     )

;; Set some part properties

    (sd-call-cmds
    (set_part_base_density
                           :parts 
                           :dens .013979) ;creates density of .505 (carbide)
     )

    (sd-call-cmds
    (set_part_inst_density
                           :parts 
                           :dens .013979) ;creates density of .505 (carbide)
     )

    (sd-call-cmds
    (set_part_base_color
                           :color
                           :modify_rgb_color
                           :rgb .5,.5,.5
                           :done
                           :done)  
     )

    (sd-call-cmds
    (set_part_inst_color
                           :color
                           :modify_rgb_color
                           :rgb .5,.5,.5
                           :done
                           :done)  
     )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;    Add BOM & other attributes     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Set part content
(setq prt_content_id (bld_prt_name "bshg" dia_1 dia_2 dp1)) 

(sd-attach-item-attribute (sd-inq-curr-part) "LHC_SPECIAL_BOM_ATTR"
                               :attachment :contents
                               :on-copy    :copy
                               :on-merge   :maintain
                               :on-split   :copy
                               :on-xform   :maintain
                               :filing     '(:sd-file :mi-file)  
                               :values     
                                           `(
                                            :material    ,"CARBIDE"
                                            :material-spec ,"LHC-12F" 
                                            :revision-level ,"_" 
                                            :part-desc   ,prt_content_id  
                                            ))

;; In order for the "change_pa_id" command to work, we must know part pathname

 (setq full_name (sd-inq-curr-part))
 (change_pa_id full_name prt_content_id) 

 (current_wp curr_wp)     ; Set workplane back to original
 (current_part curr_prt)  ; Set part back to original (since new part made)

)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Last modified    : 13-Nov-1999
;; Category         : Parts
;; Company          : L.H. Carbide Corporation
;; Author           : Bruce Fisher
;; Function Name    : make_dwl
;; Parameters       : Location point, Length of dowel, O.D., Depth.
;; Purpose          : create a dowel at selected location
;; SD Revision      : 7.51
;; Funct. Revision  : 1.30
;; Calling Dialogs  : lhc_mk_dwl
;; Comments         : 

 (defun make_dwl (A_PRT P1 DP1 D_LG DIA_D CUR_ASM D_NME)
   (setq curr_wp (sd-inq-curr-wp))
   (setq curr_prt a_prt)
   (setq pr1 (/ DIA_D 2)) ;calc radius for O.D. at top
   (setq ang_fact (tan (* (/ (* 2 pi) 360.0) 10.0))) ;calculate tan 10 in degrees
   (setq pr2 (* -1 DP1)) ;calc depth of dwl
   (setq pr5 (* -1 (+ DP1 D_LG))) ;calc end of dowel
   (setq pr3 (+ .12 pr5)) ;calc point of start diameter
   (setq pr4 (- pr1 (* .12 ang_fact))) ;calc radius of start diameter at end of dwl
   (setq pr6 (- pr1 .06))
   (setq pr7 (- pr2 .06))
   (setq rad_cnt (make-gpnt2d :x pr6 :y pr7))
   (setq rad_end1 (make-gpnt2d :x pr1 :y pr7))
   (setq rad_end2 (make-gpnt2d :x pr6 :y pr2))

     (sd-call-cmds 
       (CREATE_WORKPLANE 
                         :copy
                         :name "mk_dwl.wp"
                         :source curr_wp)) 

;; The newly created workplane is created under the root by default so
;; the root "/" has to be part of the workplane name.  It is desirable
;; to slide the origin of the workplane before rotating it so that the
;; profile building operation is easy to calculate.

    (slide_wp_origin (sd-inq-curr-wp)p1) 
    (position_wp (sd-inq-curr-wp)
           :rotate
           :axis
           :u
           :rotation_angle 90.0
           :done 
      )

;; Now build the dowel profile from user inputs

    (del_2d_geo (sd-inq-curr-wp)) ;get rid of baggage profiles first 

    (polygon
    (gpnt2d pr6 pr2)
    (gpnt2d 0 pr2)
    (gpnt2d 0 pr5)
    (gpnt2d pr4 pr5) 
    (gpnt2d pr1 pr3)
    (gpnt2d pr1 pr7))
    (arc :center rad_cnt rad_end1 rad_end2)

;; Create a new part before machining action.  
;; Need to create a unique name here.  Possibly with another function

     (sd-call-cmds 
       (CREATE_PART 
                         :name d_nme
                         :owner cur_asm   )) 

;; Run the turn tool.  

    (sd-call-cmds
    (turn  :parts d_nme
           :axis 
           :v
           :rotation_angle (* pi 2)
           :keep_wp :no
           :keep_profile :no)
     )

;; Set some part properties

    (sd-call-cmds
    (set_part_base_density
                           :parts 
                           :dens .007833) ;creates density of .283 (steel)
     )

    (sd-call-cmds
    (set_part_inst_density
                           :parts 
                           :dens .007833) ;creates density of .283 (steel)
     )

    (sd-call-cmds
    (set_part_base_color
                           :color
                           :modify_rgb_color
                           :rgb .2,.2,.2
                           :done
                           :done)  
     )

    (sd-call-cmds
    (set_part_inst_color
                           :color
                           :modify_rgb_color
                           :rgb .2,.2,.2
                           :done
                           :done)  
     )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;    Add BOM and other attributes     ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Set part content
(setq prt_content_id (bld_dwl_name "dwl" dia_d d_lg)) 

(sd-attach-item-attribute (sd-inq-curr-part) "LHC_SPECIAL_BOM_ATTR"
                               :attachment :contents
                               :on-copy    :copy
                               :on-merge   :maintain
                               :on-split   :copy
                               :on-xform   :maintain
                               :filing     '(:sd-file :mi-file)  
                               :values     
                                           `(
                                            :material    ,"PUR"
                                            :material-spec ,"*" 
                                            :part-desc   ,prt_content_id  
                                            ))


;; In order for the "change_pa_id" command to work, we must know part pathname

 (setq full_name (sd-inq-curr-part))
 (change_pa_id full_name prt_content_id) 

 (current_wp curr_wp)     ; Set workplane back to original
 (current_part curr_prt)  ; Set part back to original (since new part made)

)

